# clock - an analog/digital clock in Tcl/Tk

proc gettime {} {
  return [clock format [clock seconds] -format %H:%M:%S]
}

proc digital {} {
  proc update {} {
    .d configure -text [gettime]
    after 1000 update
  }

  label .d
  pack .d

  update
}

proc analog {} {
  set size 100
  canvas .a -width $size -height $size

  # the clock lies all inside a circle with radius $size
  #.a create oval 1 1 $size $size

  # draw the tick marks
  set pi 3.14159265358979323846
  set r [expr $size / 2] ;# radius
  set cx [set cy $r] ;# center coords
  for {set deg 0} {$deg < 360} {incr deg 6} {
    if {$deg % 30} {
      set tl 5  ;# minute tick mark length
      set color DarkGreen
    } else {
      set tl 10 ;# hour tick mark length
      set color DarkRed
    }
    set rad [expr $pi * $deg / 180]
    set cos [expr cos($rad)]
    set sin [expr sin($rad)]
    set x1 [expr $cx + $r * $sin]
    set y1 [expr $cy - $r * $cos]
    set x2 [expr $x1 - $tl * $sin]
    set y2 [expr $y1 + $tl * $cos]
    .a create line $x1 $y1 $x2 $y2 -fill $color
  }

  pack .a -padx 5 -pady 5
}

wm resizable . no no

if {[lindex $argv 0] == "-digital"} {
  digital
} else {
  analog
}
